use std::{collections::HashMap, process::exit};

use dialoguer::Select;

use crate::{services::{instagram::instagram::Instagram, service::Service, whatsapp::whatsapp::WhatsApp}, types::{participant::Participant, project::Project, user::User}};

pub async fn add(path: String) {
    let mut raw_project = Project::load(path.clone()).await.unwrap_or_else(|e| {
        eprintln!("Error while loading a project: {}", e);
        exit(1);
    });

    if raw_project.project.users.is_empty() {
        eprintln!("Error! You do not have any users in your project. Create at least one to import chat.");
        exit(1);
    }

    let services: Vec<Box<dyn Service>> = vec![
        Box::new(Instagram { 
            data: None
        }), 
        Box::new(WhatsApp { 
            data: None
        }), 
    ];

    let options: Vec<&'static str> = services.iter().map(|service| service.get_name()).collect();

    let selected = Select::new()
        .with_prompt("Choose a service")
        .items(&options)
        .default(0)
        .interact()
        .unwrap();

    let service = services.get(selected).unwrap();
    let mut final_service = service.setup();

    println!("Loading the project file");
    final_service.load().await.unwrap_or_else(|e| {
        println!("Error while loading {} export: {}", final_service.get_name(), e);
        exit(1);
    });

    let participants = final_service.get_participants();

    if participants.len() > raw_project.project.users.len() {
        eprintln!("You have {} users in your project but in the {} chat there are {} participants!", raw_project.project.users.len(), final_service.get_name(), participants.len());
        println!("You need to have equal amount or more users in project that participants in chat!");
        exit(1);
    }

    let mut user_map: HashMap<Participant, User> = HashMap::new();

    println!("Now select what project user is coresponding to a {} participant", service.get_name());
    for participant in participants {
        let users: Vec<User> = raw_project.project.users.iter().filter(|user| {
            !user_map.values().any(|x| x.name == user.name)
        }).map(|e| e.clone()).collect();

        let options: Vec<String> = users.iter().map(|usr| usr.name.clone()).collect();

        let selected = Select::new()
            .with_prompt(format!("Select a {} user for participant '{}'", service.get_name(), participant.name))
            .items(&options)
            .default(0)
            .interact()
            .unwrap();

        let selected_user = users.get(selected).unwrap().clone();

        user_map.insert(participant, selected_user);
    }

    println!("Starting to merge messages...");
    final_service.merge_messages(&mut raw_project, &user_map).await;

    raw_project.project.sort_messages();

    println!("Saving all messages to a project file.");
    let _ = raw_project.save(&path).await;
}
