use std::{collections::HashSet, process::exit};

use dialoguer::{Input, Select};
use tabled::Table;

use crate::types::{project::{Project, ProjectRaw}, user::User};

pub async fn users(path: String) {
    let project = Project::load(path.clone()).await.unwrap_or_else(|e| {
        eprintln!("Error while loading a project: {}", e);
        exit(1);
    });

    menu(project, path).await;
}

pub fn print_users(users: &Vec<User>) {
    if users.is_empty() {
        println!("Currently you do not have any users!");
        return
    }

    let users = Table::new(users).to_string();
    println!("{}", users);
}

pub async fn menu(mut project_raw: ProjectRaw, path: String) {
    let project = &mut project_raw.project;
    println!("User management:");
    print_users(&project.users);
    println!("\n");

    let options = vec!["Create a new user", "Remove a user", "Save and exit", "Exit without saving"];

    let option = Select::new()
        .with_prompt("Choose a option")
        .items(&options)
        .default(0)
        .interact()
        .unwrap();

    match option {
        0 => {
            // Create a user
            let name: String = Input::new()
                .with_prompt("Name")
                .interact_text()
                .expect("Failed to read a line");

            let user = User {
                name
            };

            project.users.push(user);

            Box::pin(menu(project_raw, path)).await;
        },
        1 => {
            if project.users.is_empty() {
                println!("You do not have any users created! No users to be removed!");
                Box::pin(menu(project_raw, path)).await;
                return;
            }

            let options_users: Vec<String> = project.users.iter().map(|user| {
                user.name.clone()
            }).collect();

            let option_remove = Select::new()
                .with_prompt("Select user to be removed")
                .items(&options_users)
                .default(0)
                .interact()
                .unwrap();

            let delete_name = options_users.get(option_remove).unwrap();
            
            let mut participants_in_chat: HashSet<String> = HashSet::new();
            for message in project.messages.clone() {
                participants_in_chat.insert(message.sender_name);

                for reaction in message.reactions {
                    participants_in_chat.insert(reaction.actor_name);
                }
            }

            if participants_in_chat.contains(delete_name) {
                eprintln!("This user has some messages in chat, therefore cannot be removed!")
            }
            
            project.users.remove(option_remove);

            Box::pin(menu(project_raw, path)).await;
        },
        2 => {
            // Save and exit
            let _ = project_raw.save(&path).await;
        },
        3 => {
            // Exit without saving
        }
        _ => {
            println!("Error! Unknown option!");
            Box::pin(menu(project_raw, path)).await;
        }
    }
}
