use std::{collections::HashMap, fs, io, path::PathBuf, process::exit};

use regex::Regex;
use async_trait::async_trait;
use dialoguer::{Input, Select};
use encoding::{all::ISO_8859_1, EncoderTrap, Encoding};
use serde::Deserialize;
use tokio::{fs::File, io::AsyncReadExt};

use crate::{services::service::Service, types::{participant::Participant, project::ProjectRaw, user::User}};

use super::types::{InstagramMessage, InstagramParticipant};

pub fn decode_str(input: &String) -> String {
    let vec = ISO_8859_1.encode(input.as_str(), EncoderTrap::Strict).unwrap();
    String::from_utf8(vec).unwrap()
}

#[derive(Deserialize, Debug)]
struct InstagramDataJson {
    messages: Vec<InstagramMessage>,
    participants: Vec<InstagramParticipant>
}

pub struct InstagramData {
    pub project_path: String,
    pub participants: Vec<InstagramParticipant>,
    pub messages: Vec<InstagramMessage>,
}

pub struct Instagram {
    pub data: Option<InstagramData>,
}

#[async_trait]
impl Service for Instagram {
    fn get_name(&self) -> &'static str {
        "Instagram"
    }

    fn setup(&self) -> Box<dyn Service> {
        let export_path: String = Input::new()
            .with_prompt("Enter path for your instagram export. (your_instagram_activity folder)")
            .interact()
            .unwrap();

        let mut path = PathBuf::from(export_path);

        path.push(PathBuf::from("messages/inbox"));

        let options: Vec<String> = fs::read_dir(&path).unwrap_or_else(|_| {
            eprintln!("Folder not found!");
            exit(1);
        })
            .filter_map(|entry| entry.ok())
            .filter(|entry| entry.path().is_dir())
            .map(|entry| { String::from(entry.file_name().to_str().unwrap()) }).collect();

        let option = Select::new()
            .with_prompt("Choose a chat")
            .items(&options)
            .default(0)
            .interact()
            .unwrap();

        let selected = options.get(option).unwrap().clone();

        path.push(selected.clone());

        Box::new(Instagram {
            data: Some(InstagramData {
                project_path: path.display().to_string(),
                participants: vec![],
                messages: vec![],
            }),
        })
    }

    async fn load(&mut self) -> io::Result<()> {
        let data = self.data.as_mut().unwrap();
        let path = PathBuf::from(&data.project_path);

        let pattern = r".*/message_.+\.json$";
        let re = Regex::new(pattern).unwrap();

        let paths: Vec<PathBuf> = fs::read_dir(&path).unwrap_or_else(|_| {
            eprintln!("Folder not found!");
            exit(1);
        })
            .filter_map(|entry| entry.ok())
            .filter(|entry| entry.path().is_file() && re.is_match(&entry.path().display().to_string()))
            .map(|entry| { entry.path() }).collect();

        for (index, path) in paths.iter().enumerate() {
            if !path.exists() {
                return Err(io::Error::new(
                    io::ErrorKind::NotFound,
                    format!(
                        "File does not exist: {}",
                        path.to_str().unwrap(),
                    ),
                ));
            }

            let mut file = File::open(path).await?;
            let mut contents = String::new();
            file.read_to_string(&mut contents).await?;

            let jsondata: InstagramDataJson = serde_json::from_str(&contents)
                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;

            if index == 0 {
                // Load participants
                for participant in jsondata.participants {
                    data.participants.push(participant);
                }
            }

            for message in jsondata.messages {
                data.messages.push(message);
            }
        }
        
        Ok(())
    }

    fn get_participants(&self) -> Vec<Participant> {
        let data = self.data.as_ref().unwrap();

        data.participants.iter().map(|participant| {
            Participant {
                name: decode_str(&participant.name)
            }
        }).collect()
    }

    async fn merge_messages(&self, raw_project: &mut ProjectRaw, user_map: &HashMap<Participant, User>) {
        let data = self.data.as_ref().unwrap();
        let path = PathBuf::from(data.project_path.clone());
        let mut media_files: Vec<(String, String)> = vec![];

        for message in &data.messages {
            let mut media_index_clone = raw_project.project.media_index.clone();
            let msg = message.conv(&mut media_index_clone, path.clone(), user_map);

            let moved = raw_project.project.push_msg(&msg);

            if moved {
                // Move media files to the media folder
                for photo in msg.photos.clone() {
                    let original_path = photo.original_img_path.unwrap().clone();
                    let path = format!("media/{}", photo.img_path);
                    let photovalues = (path, original_path);
                    media_files.push(photovalues);
                }

                for video in msg.videos.clone() {
                    let original_path = video.original_vid_path.unwrap().clone();
                    let path = format!("media/{}", video.vid_path);
                    let photovalues = (path, original_path);
                    media_files.push(photovalues);
                }

                for audio in msg.audio.clone() {
                    let original_path = audio.original_audio_path.unwrap().clone();
                    let path = format!("media/{}", audio.audio_path);
                    let photovalues = (path, original_path);
                    media_files.push(photovalues);
                }

                // Update the highest media index of the project if the message was merged
                raw_project.project.media_index = media_index_clone;
            }

        }

        let _ = raw_project.save_media_files(media_files).await;
    }
}
