use std::{collections::HashMap, path::PathBuf};
use serde::Deserialize;
use crate::types::{message::{Audio, Message, Photo, Reaction, Video}, participant::Participant, user::User};
use super::instagram::decode_str;

#[derive(Deserialize, Debug)]
pub struct InstagramParticipant {
    pub name: String,
}

#[derive(Deserialize, Debug)]
pub struct InstagramMessage {
    sender_name: String,
    timestamp_ms: usize,
    content: Option<String>,
    photos: Option<Vec<InstagramPhoto>>,
    videos: Option<Vec<InstagramVideo>>,
    audio_files: Option<Vec<InstagramAudio>>,
    reactions: Option<Vec<InstagramReaction>>
}

impl InstagramMessage {
    pub fn conv(&self, media_index: &mut usize, project_path: PathBuf, user_map: &HashMap<Participant, User>) -> Message {
        let text_content = if let Some(content) = &self.content {
            Some(decode_str(content))
        } else {
            None
        };

        let photos: Vec<Photo> = if let Some(photos_list) = &self.photos {
            photos_list.iter().map(|photo| photo.conv(&text_content, media_index, project_path.clone())).collect()
        } else {
            vec![]
        };

        let videos: Vec<Video> = if let Some(videos_list) = &self.videos {
            videos_list.iter().map(|video| video.conv(&text_content, media_index, project_path.clone())).collect()
        } else {
            vec![]
        };

        let audio: Vec<Audio> = if let Some(audio_list) = &self.audio_files {
            audio_list.iter().map(|audio| audio.conv(&text_content, media_index, project_path.clone())).collect()
        } else {
            vec![]
        };

        let reactions: Vec<Reaction> = if let Some(reaction_list) = &self.reactions {
            reaction_list.iter().map(|reaction| reaction.conv()).collect()
        } else {
            vec![]
        };

        let participant = Participant {
            name: decode_str(&self.sender_name)
        };

        let sender = user_map.get(&participant);
        let sender_name = if let Some(sender_value) = sender {
            sender_value.name.clone()
        } else {
            String::from("Unknown user")
        };

        Message {
            sender_name,
            text_content,
            timestamp: self.timestamp_ms,
            service: crate::types::message::Service::Instagram,
            photos,
            videos,
            audio,
            reactions
        }
    }
}

#[derive(Deserialize, Debug)]
pub struct InstagramReaction {
    pub reaction: String,
    pub actor: String,
    pub timestamp: Option<usize>
}

impl InstagramReaction {
    pub fn conv(&self) -> Reaction {
        Reaction {
            actor_name: decode_str(&self.actor),
            content: decode_str(&self.reaction),
            timestamp: self.timestamp.unwrap_or(0)
        }
    }
}

// Media files
#[derive(Deserialize, Debug)]
struct InstagramPhoto {
    uri: String
}
#[derive(Deserialize, Debug)]
struct InstagramVideo {
    uri: String
}
#[derive(Deserialize, Debug)]
struct InstagramAudio {
    uri: String
}
impl InstagramPhoto {
    pub fn conv(&self, text_content: &Option<String>, media_index: &mut usize, mut original_path: PathBuf) -> Photo {
        *media_index += 1;
        let img_path = (*media_index - 1).to_string();

        for _ in 0..4 {
            if !original_path.pop() {
                break;
            }
        }

        original_path.push(self.uri.clone());

        Photo {
            text_content: text_content.clone(),
            img_path,
            original_img_path: Some(original_path.display().to_string())
        }
    }
}
impl InstagramVideo {
    pub fn conv(&self, text_content: &Option<String>, media_index: &mut usize, mut original_path: PathBuf) -> Video {
        *media_index += 1;
        let vid_path = (*media_index - 1).to_string();

        for _ in 0..4 {
            if !original_path.pop() {
                break;
            }
        }

        original_path.push(self.uri.clone());

        Video {
            text_content: text_content.clone(),
            vid_path,
            original_vid_path: Some(original_path.display().to_string())
        }
    }
}
impl InstagramAudio {
    pub fn conv(&self, text_content: &Option<String>, media_index: &mut usize, mut original_path: PathBuf) -> Audio {
        *media_index += 1;
        let audio_path = (*media_index - 1).to_string();

        for _ in 0..4 {
            if !original_path.pop() {
                break;
            }
        }

        original_path.push(self.uri.clone());

        Audio {
            text_content: text_content.clone(),
            audio_path,
            original_audio_path: Some(original_path.display().to_string())
        }
    }
}

