use std::{collections::HashMap, path::PathBuf};

use serde::Deserialize;

use crate::types::{message::{Audio, Message, Photo, Video}, participant::Participant, user::User};


#[derive(Deserialize, Debug)]
pub struct WhatsAppMessage {
    pub sender: String,
    pub epoch: usize,
    pub content: Option<String>,
    pub photos: Vec<WhatsAppPhoto>,
    pub videos: Vec<WhatsAppVideo>,
    pub audio: Option<WhatsAppAudio>,
}

impl WhatsAppMessage {
    pub fn conv(&self, media_index: &mut usize, project_path: PathBuf, user_map: &HashMap<Participant, User>) -> Message {
        let photos: Vec<Photo> =
            self.photos.iter().map(|photo| photo.conv(media_index, project_path.clone())).collect();
        let videos: Vec<Video> = 
            self.videos.iter().map(|video| video.conv(media_index, project_path.clone())).collect();
        let audio: Vec<Audio> = if let Some(audio_f) = &self.audio {
            vec![audio_f.conv(media_index, project_path.clone())]
        } else {
            vec![]
        };

        let participant = Participant {
            name: self.sender.clone()
        };

        let sender = user_map.get(&participant);
        let sender_name = if let Some(sender_value) = sender {
            sender_value.name.clone()
        } else {
            String::from("Unknown user")
        };

        Message {
            sender_name,
            text_content: self.content.clone(),
            timestamp: self.epoch,
            service: crate::types::message::Service::WhatsApp,
            photos,
            videos,
            audio,
            reactions: vec![]
        }
    }
}

// Media files
#[derive(Deserialize, Debug)]
pub struct WhatsAppPhoto {
    pub uri: String,
    pub text_content: String,
}
#[derive(Deserialize, Debug)]
pub struct WhatsAppVideo {
    pub uri: String,
    pub text_content: String,
}
#[derive(Deserialize, Debug)]
pub struct WhatsAppAudio {
    pub uri: String,
    pub text_content: String,
}
impl WhatsAppPhoto {
    pub fn conv(&self, media_index: &mut usize, mut original_path: PathBuf) -> Photo {
        *media_index += 1;
        let img_path = (*media_index - 1).to_string();

        original_path.push(self.uri.clone());

        Photo {
            text_content: Some(self.text_content.clone()),
            img_path,
            original_img_path: Some(original_path.display().to_string())
        }
    }
}
impl WhatsAppVideo {
    pub fn conv(&self, media_index: &mut usize, mut original_path: PathBuf) -> Video {
        *media_index += 1;
        let vid_path = (*media_index - 1).to_string();

        original_path.push(self.uri.clone());

        Video {
            text_content: Some(self.text_content.clone()),
            vid_path,
            original_vid_path: Some(original_path.display().to_string())
        }
    }
}
impl WhatsAppAudio {
    pub fn conv(&self, media_index: &mut usize, mut original_path: PathBuf) -> Audio {
        *media_index += 1;
        let audio_path = (*media_index - 1).to_string();

        original_path.push(self.uri.clone());

        Audio {
            text_content: Some(self.text_content.clone()),
            audio_path,
            original_audio_path: Some(original_path.display().to_string())
        }
    }
}



pub struct WhatsAppParticipant {
    pub name: String
}

impl WhatsAppParticipant {
    pub fn conv(&self) -> Participant {
        Participant {
            name: self.name.clone()
        }
    }
}
